<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Api;

interface OrderDataRepositoryInterface
{
    /**
     * Save Order Data
     *
     * @param \Mediastrategi\PostNord\Api\Data\OrderDataInterface $orderData
     *
     * @return \Mediastrategi\PostNord\Api\Data\OrderDataInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function save(
        \Mediastrategi\PostNord\Api\Data\OrderDataInterface $orderdata
    );

    /**
     * Retrieve Order Data
     *
     * @param string $id
     *
     * @return \Mediastrategi\PostNord\Api\Data\OrderDataInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function get($id);

    /**
     * Retrieve PersistentQuote matching the specified criteria.
     *
     * @param \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
     *
     * @return \Magento\Framework\Api\SearchResults
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
    );

    /**
     * Delete Order Data
     *
     * @param \Mediastrategi\PostNord\Api\Data\OrderDataInterface $orderData
     *
     * @return bool true on success
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function delete(
        \Mediastrategi\PostNord\Api\Data\OrderDataInterface $orderData
    );

    /**
     * Delete Order Data by ID
     *
     * @param string $id
     *
     * @return bool true on success
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function deleteById($id);
}
