<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Cron;

class AutomationCronjob
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var \Mediastrategi\PostNord\ViewModel\Adminhtml\Automation
     */
    private $automationViewModel;

    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        \Mediastrategi\PostNord\ViewModel\Adminhtml\Automation $automationViewModel
    ) {
        $this->logger              = $logger;
        $this->automationViewModel = $automationViewModel;
    }

    /**
     * Triggers the automation process in the automation view model
     *
     * @return void
     */
    public function execute(): void
    {
        $this->logger->info(
            __('Mediastrategi PostNord Shipping Method Automation CRON started')
        );

        try {
            $this->automationViewModel->execute();
            if ($log = $this->automationViewModel->getLog()) {
                $explodes = explode("\n", $log);
                if ($explodes) {
                    foreach ($explodes as $explode) {
                        $this->logger->info($explode);
                    }
                }
            }
        } catch (\Exception $e) {
            $this->logger->info(
                __(
                    'Error: %1',
                    $e->getMessage()
                )
            );
        }

        $this->logger->info(
            __('Mediastrategi PostNord Shipping Method Automation CRON ended')
        );
    }
}
