<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Helper;

use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Exception\LocalizedException;
use Mediastrategi\PostNord\Api\Data\ScheduleInterfaceFactory;
use Mediastrategi\PostNord\Api\ScheduleRepositoryInterface;
use Mediastrategi\PostNord\Api\Data\ScheduleInterface;

class Schedule extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var ScheduleRepositoryInterface
     */
    private $scheduleRepository;
    /**
     * @var ScheduleInterfaceFactory
     */
    private $scheduleInterfaceFactory;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;
    /**
     * @var FilterBuilder
     */
    private $filterBuilder;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        ScheduleRepositoryInterface $scheduleRepository,
        ScheduleInterfaceFactory $scheduleInterfaceFactory,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        FilterBuilder $filterBuilder
    ) {
        parent::__construct($context);
        $this->scheduleRepository       = $scheduleRepository;
        $this->scheduleInterfaceFactory = $scheduleInterfaceFactory;
        $this->searchCriteriaBuilder    = $searchCriteriaBuilder;
        $this->filterBuilder            = $filterBuilder;
    }


    public function clean()
    {
        $filter = $this->filterBuilder
            ->setField(ScheduleInterface::SCHEDULE_ADDED)
            ->setConditionType('lteq')
            ->setValue(date('Y-m-d H:i:s', strtotime('-14 days')))
            ->create();

        $this->searchCriteriaBuilder->addFilters([$filter]);

        $searchCriteria = $this->searchCriteriaBuilder->create();
        try {
            $schedulesToDelete = $this->scheduleRepository->getList($searchCriteria)->getItems();
        } catch (LocalizedException $e) {
            $this->_logger->error('Could not get Schedule List');

            return; // Nothing left to do
        }

        foreach ($schedulesToDelete as $schedule) {
            try {
                $this->scheduleRepository->delete($schedule);
            } catch (LocalizedException $e) {
                $this->_logger->error('Could not delete Schedule (ID: ' . $schedule->getScheduleId() . '): '
                                      . $e->getMessage());
            }
        }
    }

    /**
     * Adds schedule
     *
     * $data is an associative array with the column names as keys
     *
     * @param  array  $data
     */
    public function add(array $data)
    {
        $newSchedule = $this->scheduleInterfaceFactory->create();
        $newSchedule->setScheduleStatus($data[ScheduleInterface::SCHEDULE_STATUS]);
        $newSchedule->setScheduleLog($data[ScheduleInterface::SCHEDULE_LOG]);

        try {
            $this->scheduleRepository->save($newSchedule);
        } catch (LocalizedException $e) {
            $this->_logger->error(
                'Could not save Schedule: ' . $e->getMessage() . ', Data: ' . print_r($data, true)
            );
        }
    }
}
