<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Model;

use Magento\Shipping\Model\Carrier\AbstractCarrierOnline;
use Magento\Shipping\Model\Carrier\CarrierInterface;

/**
 * @SuppressWarnings(PHPMD.CamelCasePropertyName)
 * phpcs:disable PSR2.Classes.PropertyDeclaration.Underscore
 */
class Carrier extends AbstractCarrierOnline implements CarrierInterface
{
    /**
     * Whether this carrier has fixed rates calculation
     *
     * @var bool
     */
    protected $_isFixed = false;

    /**
     * @var string
     */
    protected $_code = 'mspostnord';

    /**
     * @var \Mediastrategi\PostNord\Helper\Shipment
     */
    private $shipmentHelper;

    /**
     * @var \Magento\Shipping\Model\Rate\Result
     */
    private $rateResultFactory;

    /**
     * @var \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory
     */
    private $rateMethodFactory;

    /**
     * @var \Magento\Shipping\Model\Tracking\ResultFactory
     */
    private $trackFactory;

    /**
     * @var \Magento\Shipping\Model\Tracking\Result\StatusFactory
     */
    private $trackStatusFactory;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    private $productRepository;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    private $request;

    /**
     * @var \Magento\Sales\Model\Order
     */
    private $orderModel;

    /**
     * @var \Mediastrategi\PostNord\Helper\Meta
     */
    private $metaHelper;

    /**
     * @var \Mediastrategi\PostNord\Helper\PersistentQuote
     */
    private $persistentQuoteHelper;

    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory $rateErrorFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Xml\Security $xmlSecurity,
        \Magento\Shipping\Model\Simplexml\ElementFactory $xmlElFactory,
        \Magento\Shipping\Model\Tracking\Result\ErrorFactory $trackErrorFactory,
        \Magento\Directory\Model\RegionFactory $regionFactory,
        \Magento\Directory\Model\CountryFactory $countryFactory,
        \Magento\Directory\Model\CurrencyFactory $currencyFactory,
        \Magento\Directory\Helper\Data $directoryData,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        \Magento\Shipping\Model\Rate\ResultFactory $rateFactory,
        \Magento\Quote\Model\Quote\Address\RateResult\MethodFactory $rateMethodFactory,
        \Magento\Shipping\Model\Tracking\ResultFactory $trackFactory,
        \Magento\Shipping\Model\Tracking\Result\StatusFactory $trackStatusFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Mediastrategi\PostNord\Helper\Shipment $shipmentHelper,
        \Magento\Framework\App\RequestInterface $httpRequest,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Sales\Model\Order $orderModel,
        \Mediastrategi\PostNord\Helper\Meta $metaHelper,
        \Mediastrategi\PostNord\Helper\PersistentQuote $persistentQuoteHelper,
        array $data = []
    ) {
        parent::__construct(
            $scopeConfig,
            $rateErrorFactory,
            $logger,
            $xmlSecurity,
            $xmlElFactory,
            $rateFactory,
            $rateMethodFactory,
            $trackFactory,
            $trackErrorFactory,
            $trackStatusFactory,
            $regionFactory,
            $countryFactory,
            $currencyFactory,
            $directoryData,
            $stockRegistry,
            $data
        );
        $this->rateMethodFactory = $rateMethodFactory;
        $this->rateResultFactory = $rateFactory;
        $this->trackFactory = $trackFactory;
        $this->trackStatusFactory = $trackStatusFactory;
        $this->storeManager = $storeManager;
        $this->shipmentHelper = $shipmentHelper;
        $this->productRepository = $productRepository;
        $this->request = $httpRequest;
        $this->orderModel = $orderModel;
        $this->metaHelper = $metaHelper;
        $this->persistentQuoteHelper = $persistentQuoteHelper;
    }

    /**
     * @SuppressWarnings(PHPMD.CamelCaseMethodName)
     * phpcs:disable PSR2.Methods.MethodDeclaration.Underscore
     */
    protected function _doShipmentRequest(
        \Magento\Framework\DataObject $request
    ): \Magento\Framework\DataObject {
        return $this->shipmentHelper->processOrderShipmentRequest(
            $request
        );
    }

    /**
     * Processing additional validation to check is carrier applicable.
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @return false|\Mediastrategi\PostNord\Model\Carrier
     */
    public function proccessAdditionalValidation(
        \Magento\Framework\DataObject $request
    ) {
        $this->metaHelper->log(__METHOD__ . ' ' . print_r($request->debug(), true));
        return $this;
    }

    /**
     * Processing additional validation to check is carrier applicable.
     *
     * @return false|\Mediastrategi\PostNord\Model\Carrier
     */
    public function processAdditionalValidation(
        \Magento\Framework\DataObject $request
    ) {
        return $this->proccessAdditionalValidation($request);
    }

    /**
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function checkAvailableShipCountries(
        \Magento\Framework\DataObject $request
    ): \Mediastrategi\PostNord\Model\Carrier {
        return $this;
    }

    /**
     * Get Container Types, that could be customized
     *
     * @return string[]
     */
    public function getCustomizableContainerTypes(): array
    {
        return [];
    }

    /**
     * Check if carrier has shipping tracking option available
     */
    public function isTrackingAvailable(): bool
    {
        return true;
    }

    /**
     * Check if carrier has shipping label option available
     */
    public function isShippingLabelsAvailable(): bool
    {
        return true;
    }

    /**
     * Generates list of allowed carrier`s shipping methods
     * Displays on cart price rules page
     *
     * @return array
     */
    public function getAllowedMethods(): array
    {
        return [];
    }

    /**
     * Sets the number of boxes for shipping
     * PostNord only supports one package per shipment
     *
     * @param float $weight
     * @return float
     */
    public function getTotalNumOfBoxes($weight)
    {
        $this->metaHelper->log(
            (string) __(
                '%1 %2',
                __METHOD__,
                var_export($weight, true)
            )
        );
        $this->_numBoxes  = 1;
        return 1;
    }

    /**
     * Collect and get rates for storefront
     *
     * @return false|\Magento\Shipping\Model\Rate\Result
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @see \Magento\Shipping\Model\Shipping->collectCarrierRates()
     * @see \Magento\Quote\Model\Quote\Address->requestShippingRates()
     */
    public function collectRates(
        \Magento\Quote\Model\Quote\Address\RateRequest $request
    ) {
        if (!$this->metaHelper->isEnabledForCheckout()) {
            $this->metaHelper->log(
                (string) __(
                    '%1 no result since is not enabled for checkout',
                    __METHOD__
                )
            );
            return false;
        }

        // Try to extract quote from items in request
        $quote = null;
        if ($items = $request->getAllItems()) {
            foreach ($items as $item) {
                if ($quote = $item->getQuote()) {
                    break;
                }
            }
        }
        $result = false;

        // Make sure we have a quote to be able to create the correct methods
        if ($quote) {
            $this->metaHelper->log(
                (string) __('Found quote for shipping-request')
            );
            $result = $this->collectRatesForQuote(
                $request,
                $quote
            );
        } else {
            $this->metaHelper->log(
                (string) __('Did not find a quote for shipping-request')
            );
        }

        $this->metaHelper->log(
            (string) __(
                '%1 result: %2',
                __METHOD__,
                $result ? var_export($result->asArray(), true) : 'false'
            )
        );
        return $result ?? false;
    }

    private function collectRatesForQuote(
        \Magento\Quote\Model\Quote\Address\RateRequest $request,
        \Magento\Quote\Model\Quote $quote
    ): ?\Magento\Shipping\Model\Rate\Result {
        $state = [];
        $oldState = [];
        $quoteId = (int) $quote->getId();

        // Load old state here (if any)
        if (
            $oldState = $this->persistentQuoteHelper->readData(
                $quoteId
            )
        ) {
            $state = $oldState;
            $this->metaHelper->log(
                (string) __(
                    'Decoded state %1 from quote-id: %2',
                    var_export($state, true),
                    $quoteId
                )
            );
        }

        $freeBoxes =
            $this->getFreeBoxesCount($request);
        $this->setFreeBoxes($freeBoxes);
        $useFreeShipping =
            ($request->getFreeShipping() === true);
        $packageQuantity =
            $request->getPackageQty();
        $query = $this->getQueryForQuoteRateRequest(
            $request,
            $quote
        );

        $shippingOptions = $this->metaHelper->getShippingOptionsForQuery(
            $query,
            null,
            null,
            (string) $quoteId
        );
        $this->metaHelper->log(
            (string) __(
                'shipping-options: %1',
                var_export($shippingOptions, true)
            )
        );
        $result = null;

        if ($shippingOptions === null) {
            // NOTE API communication problems

            // Create one fallback method
            $methodTitle = (string) $this->metaHelper->getConfig(
                'carriers/mspostnord/fallback/title'
            );

            // Set method price
            if (
                $useFreeShipping
                || $freeBoxes == $packageQuantity
            ) {
                $price = 0.00;
            } else {
                $price = (float) $this->metaHelper->getConfig(
                    'carriers/mspostnord/fallback/cost'
                );
            }

            // Create object
            $result =
                $this->rateResultFactory->create();
            $method = $this->rateMethodFactory->create();
            /** @Var \Magento\Quote\Model\Quote\Address\RateResult\Method $method */
            $method->setCarrier($this->getCarrierCode());
            $method->setCarrierTitle($methodTitle);
            $method->setMethod($this->getCarrierCode() . '_fallback');
            $method->setMethodTitle('');
            $method->setPrice($price);
            $method->setCost($price);
            $result->append($method);
        } elseif (!empty($shippingOptions)) {
            $result =
                $this->rateResultFactory->create();
            foreach ($shippingOptions as $option) {
                if (
                    $useFreeShipping
                    || $freeBoxes == $packageQuantity
                ) {
                    $price =
                        0.00;
                } else {
                    $price =
                        $option['price'];
                }

                $optionId = $option['shipmentOptionId'];
                $availableServicePoints =
                    $option['servicePoints'] ?? [];

                /**
                 * If customer has not specified address
                 * we should clear the service points data
                 * (but not for pick up stores)
                 */
                $customerHasSpecifiedAddress = ($request->getDestCountryId()
                    && $request->getDestPostcode());
                if (
                    !$customerHasSpecifiedAddress
                    && (empty($option['serviceName'])
                        || $option['serviceName'] !== 'Pick up in store')
                ) {
                    $availableServicePoints = [];
                }

                $firstServicePoint =
                    $availableServicePoints
                    ? current($availableServicePoints)
                    : [];
                $selectedServicePoint = [];
                $availableAddons =
                    $option['additionalServices'] ?? [];
                $selectedAddons = [];

                // Determine shipping-option service selected point
                if (
                    isset(
                        $state[$optionId]['selectedServicePoint'],
                        $state[$optionId]['selectedServicePoint']['servicePointId']
                    )
                ) {
                    if ($availableServicePoints) {
                        $foundSelectedServicePoint = false;
                        foreach ($availableServicePoints as $availableServicePoint) {
                            if (
                                !empty($availableServicePoint['servicePointId'])
                                && $availableServicePoint['servicePointId'] ==
                                $state[$optionId]['selectedServicePoint']['servicePointId']
                            ) {
                                $foundSelectedServicePoint = true;
                                break;
                            }
                        }
                        if (!$foundSelectedServicePoint) {
                            $state[$optionId]['selectedServicePoint'] =
                                $firstServicePoint;
                        }
                    } else {
                        $state[$optionId]['selectedServicePoint'] =
                            [];
                    }
                } else {
                    if ($availableServicePoints) {
                        $state[$optionId]['selectedServicePoint'] =
                            $firstServicePoint;
                    } else {
                        $state[$optionId]['selectedServicePoint'] =
                            [];
                    }
                }
                $selectedServicePoint =
                    $state[$optionId]['selectedServicePoint'];

                // Determine selected add-ons on option
                if (isset($state[$optionId]['selectedAddons'])) {
                    if ($availableAddons) {
                        foreach (array_keys($state[$optionId]['selectedAddons']) as $selectedAddonKey) {
                            $foundSelectedAddon = false;
                            foreach ($availableAddons as $availableAddon) {
                                if (
                                    isset($availableAddon['id'])
                                    && (string) $selectedAddonKey
                                    === $availableAddon['id']
                                ) {
                                    $foundSelectedAddon = true;
                                    break;
                                }
                            }
                            if (!$foundSelectedAddon) {
                                unset(
                                    $state[$optionId]['selectedAddons'][$selectedAddonKey]
                                );
                            }
                        }
                    }
                } else {
                    $state[$optionId]['selectedAddons'] =
                        [];
                }
                $selectedAddons =
                    $state[$optionId]['selectedAddons'];

                $method = $this->rateMethodFactory->create();
                /** @var \Magento\Quote\Model\Quote\Address\RateResult\Method $method */
                $method->setCarrier($this->getCarrierCode());
                $method->setCarrierTitle($option['deliveryMethod']);
                $method->setMethod($option['shipmentOptionId']);
                $method->setMethodTitle($option['description']);
                $method->setPrice($price);
                $method->setCost($price);
                $method->setMspostnordShipmentOptionId($optionId);
                $method->setMspostnordSelectedServicePoint(
                    json_encode($selectedServicePoint)
                );
                $method->setMspostnordAvailableServicePoints(
                    json_encode($availableServicePoints)
                );
                $method->setMspostnordSelectedAddons(
                    json_encode($selectedAddons)
                );
                $method->setMspostnordAvailableAddons(
                    json_encode($availableAddons)
                );
                $method->setMspostnordLongDescription(
                    $option['longDescription']
                );
                $method->setMspostnordIsRecommended(
                    !empty($option['isRecommended'])
                );
                $method->setMspostnordDeliveryEstimateText(
                    $option['deliveryEstimateText'] ?: ''
                );
                $result->append($method);

                /**
                 * NOTE: method is converted via
                 * \Magento\Quote\Model\Cart\ShippingMethodConverter->modelToDataObject()
                 * before delivered to front-end
                 */
            }
        }

        if ($oldState !== $state) {
            $this->metaHelper->log(
                (string) __(
                    'State has changed, old: "%1" vs new: "%2", saving..',
                    var_export($oldState, true),
                    var_export($state, true)
                )
            );
            $this->persistentQuoteHelper->update(
                $quoteId,
                $state
            );
        } else {
            $this->metaHelper->log(
                (string) __(
                    'State has not changed'
                )
            );
        }

        return $result ?? null;
    }

    /**
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getQueryForQuoteRateRequest(
        \Magento\Quote\Model\Quote\Address\RateRequest $request,
        \Magento\Quote\Model\Quote $quote
    ): array {
        /**
         * Some modules set request->countryId to
         * shipping origin so if request->destCountryId
         * is not set we instead use shop default country
         * since it's more logical
         */
        /**
         * Check quote address if we have a company defined
         * if so set type to 'business' otherwise to 'personal'
         */
        $customerType = 'personal';
        foreach ($quote->getAddressesCollection() as $address) {
            /** @var \Magento\Quote\Model\Quote\Address $address */
            if ($address->getCompany()) {
                $customerType = 'business';
                break;
            }
        }

        $weightInGrams = 0;
        $packageWeight =
            (float) $request->getPackageWeight();

        // Set minimum weight here
        $minimumWeight = (float) $this->metaHelper->getConfig(
            'carriers/mspostnord/weight/products_minimum_weight'
        );
        if ($packageWeight < $minimumWeight) {
            $packageWeight = $minimumWeight;
        }

        // Add wrapping weight here
        $wrappingWeight = (float) $this->metaHelper->getConfig(
            'carriers/mspostnord/weight/wrapping_weight'
        );
        $packageWeight += $wrappingWeight;

        // Convert package-weight to grams
        $weightUnit = $this->metaHelper->getConfig(
            'general/locale/weight_unit'
        );
        if ($weightUnit === 'lbs') {
            $weightInGrams =
                $packageWeight * 453.5924;
        } elseif ($weightUnit === 'kgs') {
            $weightInGrams =
                $packageWeight * 1000;
        } else {
            throw new \Magento\Framework\Exception\LocalizedException(
                __(
                    'Unexpected weight unit %1!',
                    $weightUnit
                )
            );
        }

        $query = [
            'currency' =>
                (string) $this->storeManager->getStore()
                    ->getBaseCurrency()
                    ->getCurrencyCode(),
            'language' =>
                (string) $this->metaHelper->getLanguageCode(),
            'recipient' => [
                'countryCode' =>
                    (string) $request->getDestCountryId()
                    ?: $this->metaHelper->getConfig(
                        'general/country/default'
                    )
                    ,
                'postalCode' =>
                    (string) $request->getDestPostcode()
                    ?: $request->getPostcode(),
                'type' =>
                    $customerType,
            ],
            'totalPrice' => (int) round(
                (float) $request->getBaseSubtotalInclTax(),
                2
            ) * 100,
            'weightInGrams' =>
                (int) $weightInGrams,
        ];

        $this->metaHelper->log(
            sprintf(
                '%s calculated query from shipping request: %s to %s',
                __METHOD__,
                print_r(
                    $request->debug(),
                    true
                ),
                var_export($query, true)
            )
        );
        return $query;
    }

    /**
     * @param  mixed  $item
     */
    private function getFreeBoxesCountFromChildren($item): int
    {
        $freeBoxes = 0;
        foreach ($item->getChildren() as $child) {
            if (
                $child->getFreeShipping()
                && !$child->getProduct()->isVirtual()
            ) {
                $freeBoxes += $item->getQty() * $child->getQty();
            }
        }

        return $freeBoxes;
    }

    /**
     * @param string|array<int,string> $trackings
     */
    public function getTracking(
        $trackings
    ): \Magento\Shipping\Model\Tracking\Result {
        $this->metaHelper->log(
            (string) __(
                '%1 $trackings: %2',
                __METHOD__,
                var_export($trackings, true)
            )
        );
        if (!is_array($trackings)) {
            $trackings = [$trackings];
        }
        $result =
            $this->trackFactory->create();
        foreach ($trackings as $tracking) {
            if (
                $trackingUrl = $this->shipmentHelper->getExternalShipmentTrackingLink(
                    $tracking
                )
            ) {
                $status = $this->trackStatusFactory->create();
                $status->setTracking($tracking);
                $status->setPopup(1);
                $status->setUrl($trackingUrl);
                $result->append($status);
            } else {
                $error = $this->_trackErrorFactory->create();
                $error->setTracking($tracking);
                $error->setErrorMessage(
                    __(
                        "Shipment with consignment number '%1' couldn't be found",
                        $tracking
                    )
                );
                $result->append($error);
            }
        }
        return $result;
    }

    private function getFreeBoxesCount(
        \Magento\Quote\Model\Quote\Address\RateRequest $request
    ): int {
        $freeBoxes = 0;
        if ($request->getAllItems()) {
            foreach ($request->getAllItems() as $item) {
                if (
                    $item->getProduct()->isVirtual()
                    || $item->getParentItem()
                ) {
                    continue;
                }

                if (
                    $item->getHasChildren()
                    && $item->isShipSeparately()
                ) {
                    $freeBoxes += $this->getFreeBoxesCountFromChildren($item);
                } elseif ($item->getFreeShipping()) {
                    $freeBoxes += $item->getQty();
                }
            }
        }
        return $freeBoxes;
    }
}
