<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Model;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Api\ExtensibleDataObjectConverter;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Store\Model\StoreManagerInterface;
use Mediastrategi\PostNord\Api\Data\ScheduleInterfaceFactory;
use Mediastrategi\PostNord\Api\Data\ScheduleSearchResultsInterfaceFactory;
use Mediastrategi\PostNord\Api\ScheduleRepositoryInterface;
use Mediastrategi\PostNord\Model\ResourceModel\Schedule as ResourceSchedule;
use Mediastrategi\PostNord\Model\ResourceModel\Schedule\CollectionFactory as ScheduleCollectionFactory;

class ScheduleRepository implements ScheduleRepositoryInterface
{
    protected $extensibleDataObjectConverter;
    protected $resource;

    protected $searchResultsFactory;

    protected $scheduleCollectionFactory;

    private $storeManager;

    protected $dataObjectHelper;

    protected $scheduleFactory;

    protected $dataScheduleFactory;

    protected $dataObjectProcessor;

    protected $extensionAttributesJoinProcessor;

    private $collectionProcessor;


    /**
     * @param  ResourceSchedule  $resource
     * @param  ScheduleFactory  $scheduleFactory
     * @param  ScheduleInterfaceFactory  $dataScheduleFactory
     * @param  ScheduleCollectionFactory  $scheduleCollectionFactory
     * @param  ScheduleSearchResultsInterfaceFactory  $searchResultsFactory
     * @param  DataObjectHelper  $dataObjectHelper
     * @param  DataObjectProcessor  $dataObjectProcessor
     * @param  StoreManagerInterface  $storeManager
     * @param  CollectionProcessorInterface  $collectionProcessor
     * @param  JoinProcessorInterface  $extensionAttributesJoinProcessor
     * @param  ExtensibleDataObjectConverter  $extensibleDataObjectConverter
     */
    public function __construct(
        ResourceSchedule $resource,
        ScheduleFactory $scheduleFactory,
        ScheduleInterfaceFactory $dataScheduleFactory,
        ScheduleCollectionFactory $scheduleCollectionFactory,
        ScheduleSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter
    ) {
        $this->resource                         = $resource;
        $this->scheduleFactory                  = $scheduleFactory;
        $this->scheduleCollectionFactory        = $scheduleCollectionFactory;
        $this->searchResultsFactory             = $searchResultsFactory;
        $this->dataObjectHelper                 = $dataObjectHelper;
        $this->dataScheduleFactory              = $dataScheduleFactory;
        $this->dataObjectProcessor              = $dataObjectProcessor;
        $this->storeManager                     = $storeManager;
        $this->collectionProcessor              = $collectionProcessor;
        $this->extensionAttributesJoinProcessor = $extensionAttributesJoinProcessor;
        $this->extensibleDataObjectConverter    = $extensibleDataObjectConverter;
    }

    /**
     * {@inheritdoc}
     */
    public function save(
        \Mediastrategi\PostNord\Api\Data\ScheduleInterface $schedule
    ) {
        /* if (empty($schedule->getStoreId())) {
            $storeId = $this->storeManager->getStore()->getId();
            $schedule->setStoreId($storeId);
        } */

        $scheduleData = $this->extensibleDataObjectConverter->toNestedArray(
            $schedule,
            [],
            \Mediastrategi\PostNord\Api\Data\ScheduleInterface::class
        );

        $scheduleModel = $this->scheduleFactory->create()->setData($scheduleData);

        try {
            $this->resource->save($scheduleModel);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the schedule: %1',
                $exception->getMessage()
            ));
        }

        return $scheduleModel->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function get($scheduleId)
    {
        $schedule = $this->scheduleFactory->create();
        $this->resource->load($schedule, $scheduleId);
        if (!$schedule->getId()) {
            throw new NoSuchEntityException(__('Schedule with id "%1" does not exist.', $scheduleId));
        }

        return $schedule->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->scheduleCollectionFactory->create();

        $this->extensionAttributesJoinProcessor->process(
            $collection,
            \Mediastrategi\PostNord\Api\Data\ScheduleInterface::class
        );

        $this->collectionProcessor->process($criteria, $collection);

        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);

        $items = [];
        foreach ($collection as $model) {
            $items[] = $model->getDataModel();
        }

        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());

        return $searchResults;
    }

    /**
     * {@inheritdoc}
     */
    public function delete(
        \Mediastrategi\PostNord\Api\Data\ScheduleInterface $schedule
    ) {
        try {
            $scheduleModel = $this->scheduleFactory->create();
            $this->resource->load($scheduleModel, $schedule->getScheduleId());
            $this->resource->delete($scheduleModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the Schedule: %1',
                $exception->getMessage()
            ));
        }

        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function deleteById($scheduleId)
    {
        return $this->delete($this->get($scheduleId));
    }
}
