<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Model;

use Magento\Framework\Api\DataObjectHelper;
use Mediastrategi\PostNord\Api\Data\TransactionInterface;
use Mediastrategi\PostNord\Api\Data\TransactionInterfaceFactory;

/**
 * @SuppressWarnings(PHPMD.CamelCasePropertyName)
 * phpcs:disable PSR2.Classes.PropertyDeclaration.Underscore
 */
class Transaction extends \Magento\Framework\Model\AbstractModel
{
    protected $_eventPrefix = 'mediastrategi_postnord_transaction';
    protected $dataObjectHelper;
    protected $transactionDataFactory;

    /**
     * @param  \Magento\Framework\Model\Context  $context
     * @param  \Magento\Framework\Registry  $registry
     * @param  TransactionInterfaceFactory  $transactionDataFactory
     * @param  DataObjectHelper  $dataObjectHelper
     * @param  \Mediastrategi\PostNord\Model\ResourceModel\Transaction  $resource
     * @param  \Mediastrategi\PostNord\Model\ResourceModel\Transaction\Collection  $resourceCollection
     * @param  array  $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        TransactionInterfaceFactory $transactionDataFactory,
        DataObjectHelper $dataObjectHelper,
        \Mediastrategi\PostNord\Model\ResourceModel\Transaction $resource,
        \Mediastrategi\PostNord\Model\ResourceModel\Transaction\Collection $resourceCollection,
        array $data = []
    ) {
        $this->transactionDataFactory = $transactionDataFactory;
        $this->dataObjectHelper       = $dataObjectHelper;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * Retrieve transaction model with transaction data
     * @return TransactionInterface
     */
    public function getDataModel()
    {
        $transactionData = $this->getData();

        $transactionDataObject = $this->transactionDataFactory->create();
        $this->dataObjectHelper->populateWithArray(
            $transactionDataObject,
            $transactionData,
            TransactionInterface::class
        );

        return $transactionDataObject;
    }
}
