<?php

declare(strict_types=1);

namespace Mediastrategi\PostNord\Test\Unit;

class OrderDataTest extends \PHPUnit\Framework\TestCase
{
    /**
     * @var \Mediastrategi\PostNord\Helper\OrderData
     */
    private $orderDataHelper;

    protected function setUp(): void
    {
        $objectManager =
            new \Magento\Framework\TestFramework\Unit\Helper\ObjectManager($this);
        $orderData = $this->createPartialMock(
            \Mediastrategi\PostNord\Api\Data\OrderDataInterface::class,
            [
                'getExtensionAttributes',
                'getId',
                'getOrderData',
                'getOrderId',
                'setExtensionAttributes',
                'setId',
                'setOrderData',
                'setOrderId',
            ]
        );
        $orderData
        ->method('getExtensionAttributes')
            ->will($this->returnValue([]));
        $orderData
        ->method('getId')
            ->will($this->returnValue(1));
        $orderData
        ->method('getOrderData')
            ->will($this->returnValue(json_encode(["a" => 123, "b" => 456])));
        $orderData
        ->method('getOrderId')
            ->will($this->returnValue(1));
        $orderData
        ->method('setExtensionAttributes')
            ->will($this->returnValue($orderData));
        $orderData
        ->method('setId')
            ->will($this->returnValue($orderData));
        $orderData
        ->method('setOrderData')
            ->will($this->returnValue($orderData));
        $orderData
        ->method('setOrderId')
            ->will($this->returnValue($orderData));

        $orderDataRepository = $this->createPartialMock(
            \Mediastrategi\PostNord\Api\OrderDataRepositoryInterface::class,
            [
                'delete',
                'deleteById',
                'get',
                'getList',
                'save'
            ]
        );
        $orderDataRepository
        ->method('delete')
            ->will($this->returnValue(true));
        $orderDataRepository
        ->method('deleteById')
            ->will($this->returnValue(true));
        $orderDataRepository
        ->method('get')
            ->will($this->returnValue($orderData));
        $orderDataRepository
        ->method('getList')
            ->will($this->returnValue([$orderData]));
        $orderDataRepository
        ->method('save')
            ->will($this->returnValue(true));

        $orderDataFactory = $this->createPartialMock(
            \Mediastrategi\PostNord\Api\Data\OrderDataInterfaceFactory::class,
            ['create']
        );
        $orderDataFactory->expects($this->any())
            ->method('create')
            ->will($this->returnValue($orderData));

        $searchCriteria = $this->createMock(
            \Magento\Framework\Api\SearchCriteriaInterface::class
        );
        $searchCriteriaBuilder = $this->createPartialMock(
            \Magento\Framework\Api\SearchCriteriaBuilder::class,
            ['create']
        );
        $searchCriteriaBuilder->expects($this->any())
            ->method('create')
            ->will($this->returnValue($searchCriteria));

        $orderDataModel = $this->createPartialMock(
            \Mediastrategi\PostNord\Model\OrderData::class,
            [
                'getDataModel',
                'getId',
            ]
        );
        $orderDataModel
        ->method('getDataModel')
            ->will($this->returnValue($orderData));
        $orderDataModel
        ->method('getId')
            ->will($this->returnValue(1));

        $orderDataCollection = $this->createPartialMock(
            \Mediastrategi\PostNord\Model\ResourceModel\OrderData\Collection::class,
            [
                'addFieldToFilter',
                'getFirstItem'
            ]
        );
        $orderDataCollection
        ->method('addFieldToFilter')
            ->will($this->returnValue($orderDataCollection));
        $orderDataCollection
        ->method('getFirstItem')
            ->will($this->returnValue($orderDataModel));

        $orderDataCollectionFactory = $this->createPartialMock(
            \Mediastrategi\PostNord\Model\ResourceModel\OrderData\CollectionFactory::class,
            [
                'create'
            ]
        );
        $orderDataCollectionFactory
        ->method('create')
            ->will($this->returnValue($orderDataCollection));

        $this->orderDataHelper = $objectManager->getObject(
            \Mediastrategi\PostNord\Helper\OrderData::class,
            [
                'orderDataCollectionFactory' =>
                    $orderDataCollectionFactory,
                'orderDataFactory' =>
                    $orderDataFactory,
                'orderDataRepository' =>
                    $orderDataRepository,
                'searchCriteriaBuilder' =>
                    $searchCriteriaBuilder,
            ]
        );
    }

    public function testCreate()
    {
        // Prep
        $orderData = [
            'attr'  => 'data',
            'attr2' => 'data2',
        ];

        // Action
        $result = $this->orderDataHelper->create(
            1,
            $orderData
        );

        // Assert
        $this->assertTrue($result);
    }

    public function testRead()
    {
        $result = $this->orderDataHelper->read(1);
        $this->assertNotNull($result);
    }

    public function testReadData()
    {
        $result = $this->orderDataHelper->readData(1);
        $this->assertEquals(
            $result,
            ["a" => 123, "b" => 456]
        );
    }

    public function testUpdate()
    {
        // Prep
        $orderData = [
            'attr'  => 'data',
            'attr2' => 'data3',
        ];
        $this->assertTrue(
            $this->orderDataHelper->update(
                1,
                $orderData
            )
        );
    }

    public function testDelete()
    {
        $this->assertTrue(
            $this->orderDataHelper->delete(1)
        );
    }
}
