/* global require */
require(['jquery'], function($) {
    $(document).ready(function() {

        /**
         * @var {String}
         */
        var mspostnord_ajaxUrl = '';

        /**
         * @var {Boolean}
         */
        var mspostnord_isRefreshing = false;

        /**
         * @var {Boolean}
         */
        var mspostnord_isSaving = false;

        /**
         * @var {Number}
         */
        var mspostnord_orderId = 0;

        /**
         * @var {Object}
         */
        var mspostnord_i18n = {};

        /**
         * Refresh available shipment options for order via XHR
         * do it at most one time at a time.
         */
        function mspostnord_refresh()
        {
            if (!mspostnord_isRefreshing) {
                mspostnord_isRefreshing = true;
                $('#mspostnord-information-admintml-order-view').addClass('is-loading');
                $.ajax(
                    mspostnord_ajaxUrl,
                    {
                        data: {
                            action:
                                'refresh',
                            orderId:
                                mspostnord_orderId
                        },
                        dataType: 'json',
                        error: function (response) {
                            $('#mspostnord-information-admintml-order-view').removeClass('is-loading');
                            mspostnord_isRefreshing = false;
                        },
                        success: function(response) {
                            mspostnord_debug(function() {
                                console.log('Refresh response:');
                                console.log(response);
                            });
                            $('#mspostnord-information-admintml-order-view').removeClass('is-loading');
                            var session = {
                                additionalServices: {},
                                selectedServicePointId: 0,
                                selectedShipmentOptionId: 0
                            };
                            if (typeof(response.session) !== 'undefined') {
                                session =
                                    response.session;
                            }
                            if (typeof(response.options) !== 'undefined') {
                                var option, optionHtml, shipmentOptionIsSelected, servicePointIsSelected;
                                optionHtml = '<div class="m2-postnord-widget is-loading">';

                                if (response.options.length === 0) {
                                    optionHtml += '<p class="no-options">' + mspostnord_i18n['Found no shipping-options for order!'] + '</p>';
                                } else {
                                    optionHtml += '<div class="shipping-options">';
                                    optionHtml += '<p class="select-shipping-option">' + mspostnord_i18n['Select Shipping-option'] + '</p>';
                                    optionHtml += '<ul class="shipping-options">';
                                    for (var optionIndex in response.options) {
                                        if (response.options.hasOwnProperty(optionIndex)) {
                                            option = response.options[optionIndex];
                                            shipmentOptionIsSelected =
                                                option.shipmentOptionId === session.selectedShipmentOptionId;
                                            optionHtml += '<li class="shipping-option ' + (shipmentOptionIsSelected ? 'selected' : 'not-selected')
                                                + ' ' + (option.longDescription !== '' ? 'with-tooltip' : 'without-tooltip')
                                                + ' hide-tooltip"><label>';
                                            optionHtml += '<input type="radio" required="required" class="shipping-method-radio" name="mspostnord_new_shipment_option" value="' + option.shipmentOptionId + '" ' + (shipmentOptionIsSelected ? ' checked="checked"' : '') + '/><div class="radio-body">';
                                            var isRecommended =
                                                typeof(option.isRecommended) !== 'undefined'
                                                && option.isRecommended === true;
                                            if (isRecommended) {
                                                optionHtml += '<span class="is-recommended">' + mspostnord_i18n['is_recommended'] + '</span>';
                                            }

                                            optionHtml += '<div class="name-wrapper">';
                                            optionHtml += '<strong class="name">' + option.deliveryMethod;
                                            if (option.longDescription !== '') {
                                                optionHtml += '<span class="tooltip-question-mark"></span>';
                                            }
                                            optionHtml += '</strong>';
                                            if (option.longDescription !== '') {
                                                optionHtml += '<div class="tooltip-wrapper"><span class="tooltip-text">'
                                                    + option.longDescription + '</span>'
                                                    + '<span class="tooltip-pointer-1"></span>'
                                                    + '<span class="tooltip-pointer-2"></span>'
                                                    + '</div>';
                                            }

                                            optionHtml += '</div>';
                                            optionHtml += '<span class="option-description">' + option.description + '</span>';

                                            if (option.deliveryEstimateText !== '') {
                                                optionHtml += '<p class="eta">'
                                                    + '<strong class="eta-heading">' + mspostnord_i18n['delivery_text'] + '</strong>'
                                                    + '<span class="eta-value">' + option.deliveryEstimateText + '</span>'
                                                    + '</p>';
                                            }

                                            optionHtml += '</div></label><div class="optionals">';
                                            if (typeof(option.servicePoints) !== 'undefined'
                                                && option.servicePoints
                                                && option.servicePoints.length > 0
                                               ) {
                                                optionHtml += '<div class="service-points"><ul>';
                                                var isFirstServicePoint = true;
                                                for (var servicePointIndex in option.servicePoints)
                                                {
                                                    if (option.servicePoints.hasOwnProperty(servicePointIndex)) {
                                                        var servicePoint =
                                                            option.servicePoints[servicePointIndex];
                                                        if (session.selectedServicePointId) {
                                                            servicePointIsSelected =
                                                                servicePoint.servicePointId === session.selectedServicePointId;
                                                        } else {
                                                            servicePointIsSelected =
                                                                isFirstServicePoint;
                                                        }
                                                        if (typeof(servicePoint.servicePointId) !== 'undefined'
                                                            && servicePoint.servicePointId
                                                            && typeof(servicePoint.formattedName) !== 'undefined'
                                                           ) {
                                                            optionHtml += '<li class="service-point ' + (servicePointIsSelected ? 'selected' : 'not-selected') + '"><label><input type="radio" class="service-point-radio" name="mspostnord_service_point" value="' + servicePoint.servicePointId + '" data-formatted-name="' + servicePoint.formattedName + '"' + (servicePointIsSelected ? 'checked="checked"' : '') + ' /><div class="radio-body"><strong class="name">' + servicePoint.name + '</strong>';
                                                            var servicePointAddress =
                                                                servicePoint.visitingAddress.streetName;
                                                            if (
                                                                typeof(servicePoint.visitingAddress.streetNumber) !== 'undefined'
                                                            ) {
                                                                servicePointAddress +=
                                                                    ' ' + servicePoint.visitingAddress.streetNumber;
                                                            }
                                                            servicePointAddress +=
                                                                ' ' + servicePoint.visitingAddress.postalCode
                                                                + ' ' + servicePoint.visitingAddress.city
                                                                + ' ' + servicePoint.visitingAddress.countryCode;
                                                            optionHtml += '<span class="service-point-address">' + servicePointAddress + '</span></div>'
                                                                + '</label></li>';
                                                        }
                                                        if (isFirstServicePoint) {
                                                            isFirstServicePoint = false;
                                                        }
                                                    }
                                                }
                                                optionHtml += '</ul></div>';
                                            }

                                            if (
                                                typeof(option.additionalServices) !== 'undefined'
                                                && option.additionalServices
                                                && option.additionalServices.length > 0
                                            ) {
                                                optionHtml += '<div class="additional-services">'
                                                    + '<ul>';
                                                for (var additionalServiceKey in option.additionalServices) {
                                                    if (option.additionalServices.hasOwnProperty(additionalServiceKey)) {
                                                        var additionalService =
                                                            option.additionalServices[additionalServiceKey];
                                                        var additionalServiceSelectedValue =
                                                            '';
                                                        if (additionalService.type === 'checkbox') {
                                                            additionalServiceSelectedValue =
                                                                typeof(session.additionalServices) !== 'undefined'
                                                                && typeof(session.additionalServices[additionalService.id]) !== 'undefined'
                                                                ? session.additionalServices[additionalService.id]
                                                                : '';
                                                            optionHtml += '<li class="additional-service ' + (additionalServiceSelectedValue ? 'selected' : 'not-selected') + '"><label>'
                                                                + '<input class="additional-service-checkbox" type="checkbox" value="'
                                                                + additionalService.id + '"'
                                                                + (additionalServiceSelectedValue ? ' checked="checked"' : '')
                                                                + ' /><div class="radio-body">'
                                                                + '<strong class="name">' + additionalService.name + '</strong>'
                                                                + '<span class="addon-description">' + additionalService.helpText
                                                                + '</span></label></div></li>';
                                                        } else if (additionalService.type === 'doorCode') {
                                                            additionalServiceSelectedValue =
                                                                typeof(session.additionalServices) !== 'undefined'
                                                                && typeof(session.additionalServices[additionalService.id]) !== 'undefined'
                                                                && typeof(session.additionalServices[additionalService.id].value) !== 'undefined'
                                                                ? session.additionalServices[additionalService.id].value
                                                                : '';
                                                            optionHtml += '<li class="additional-service"><label><strong class="name">' + additionalService.name + '</strong>';
                                                            if (additionalService.helpText !== additionalService.name) {
                                                                optionHtml += '<span class="addon-description">' + additionalService.helpText + '</span>';
                                                            }
                                                            optionHtml += '<input class="additional-service-doorcode" name="'
                                                                + additionalService.id + '" type="text" value="'
                                                                + additionalServiceSelectedValue + '" /></label></li>';
                                                        }
                                                    }
                                                }
                                                optionHtml += '</ul></div>';
                                            }

                                            optionHtml += '</div</li>';
                                        }
                                    }
                                }

                                $('#mspostnord-information-admintml-order-view .mspostnord-change-shipping-method .widget-container').replaceWith(optionHtml);

                                $('#mspostnord-information-admintml-order-view .mspostnord-change-shipping-method .shipping-option.with-tooltip .name-wrapper > .name > .tooltip-question-mark').click(function(event) {
                                    event.preventDefault();
                                    var isMobile =
                                        ($(window).width() <= 540);
                                    var ref =
                                        $(this).parents('.shipping-option').first();
                                    if (ref.hasClass('show-tooltip')) {
                                        $('.shipping-option.with-tooltip.show-tooltip').removeClass('show-tooltip').addClass('hide-tooltip');
                                    } else {
                                        if (isMobile) {
                                            var tooltip =
                                                $('.tooltip-wrapper', $(this).parent().parent()).first().css('top', '');
                                        } else {
                                            // Adjust tooltip position
                                            var tooltip =
                                                $('.tooltip-wrapper', $(this).parent().parent()).first();
                                            var newTop =
                                                22 - 1 * Math.round(tooltip.outerHeight() / 2);
                                            $(tooltip).css('top', newTop);

                                            // Adjust tooltip pointers position
                                            var tooltipPointer1 =
                                                $('.tooltip-pointer-1', $(this).parent().parent()).first();
                                            var tooltipPointer2 =
                                                $('.tooltip-pointer-2', $(this).parent().parent()).first();
                                            var newTop1 =
                                                Math.round(tooltip.outerHeight() / 2) - 6;
                                            $(tooltipPointer1).css('top', newTop1);
                                            $(tooltipPointer2).css('top', newTop1 - 1);
                                        }

                                        $('.shipping-option.with-tooltip.show-tooltip').removeClass('show-tooltip').addClass('hide-tooltip');
                                        ref.addClass('show-tooltip').removeClass('hide-tooltip');
                                    }
                                });

                                // Update CSS classes when selecting shipping option
                                $('#mspostnord-information-admintml-order-view .shipping-method-radio').click(function(event) {
                                    $('#mspostnord-information-admintml-order-view .shipping-option.selected').removeClass('selected').addClass('not-selected');
                                    $(this).parent().parent().removeClass('not-selected').addClass('selected');
                                });

                                // Update CSS classes when selecting service point
                                $('#mspostnord-information-admintml-order-view .service-point-radio').click(function(event) {
                                    var parent = $(this).parent().parent().parent();
                                    $('.service-point-radio.selected', parent).removeClass('selected').addClass('not-selected');
                                    $(this).parent().parent().removeClass('not-selected').addClass('selected');
                                });
                                $('#mspostnord-information-admintml-order-view .m2-postnord-widget').removeClass('is-loading');

                                // Enable save button if we have a pre-selected option
                                if (session.selectedShipmentOptionId) {
                                    $('button.action-search-save').removeAttr('disabled');
                                }

                                // Disable button to load options
                                $('button.action-search-method').attr('disabled', 'disabled');

                                // Change shipment option
                                $('#mspostnord-information-admintml-order-view .shipping-method-radio').click(function(event) {
                                    $('button.action-search-save').removeAttr('disabled');
                                });
                            }

                            mspostnord_isRefreshing = false;
                        },
                        method: 'GET'
                    }
                );
            }
        }

        /**
         * Save shipment data on order, then reload page.
         * Do it at most one time at a time.
         *
         * @param {Object} option
         */
        function mspostnord_save(option)
        {
            if (!mspostnord_isSaving) {
                mspostnord_isSaving = true;
                $('#mspostnord-information-admintml-order-view').addClass('is-loading');
                var selected =
                    $('.m2-postnord-widget .shipping-method-radio:checked');
                if (selected.length) {
                    selected =
                        selected.parent().parent();
                    var carrierTitle =
                        $('.name:first', selected).text();
                    carrierTitle =
                        carrierTitle ? carrierTitle : '';
                    var methodTitle =
                        $('.option-description', selected).text();
                    methodTitle =
                        methodTitle ? methodTitle : '';
                    var shipmentOptionId =
                        $('.shipping-method-radio', selected).val();
                    shipmentOptionId =
                        shipmentOptionId ? shipmentOptionId : 0;
                    var servicePointId =
                        $('.service-point-radio:checked', selected).val();
                    servicePointId =
                        servicePointId ? servicePointId : 0;
                    var servicePointName =
                        $('.service-point-radio:checked', selected).attr('data-formatted-name');
                    servicePointName =
                        servicePointName ? servicePointName : '';

                    // Build additional services here
                    var additionalServices = {};
                    if ($('.additional-service-checkbox:checked', selected).length) {
                        $('.additional-service-checkbox:checked', selected).each(function(i, object) {
                            var helpText = '';
                            if ($('.addon-description', $(object).parent()).length) {
                                helpText =
                                    $('.addon-description', $(object).parent()).text();
                            }
                            var id = $(object).attr('value');
                            var name = '';
                            if ($('.name', $(object).parent()).length) {
                                name =
                                    $('.name', $(object).parent()).text();
                            }
                            var value = true;
                            additionalServices[id] = {
                                helpText: helpText,
                                id: id,
                                name: name,
                                value: value
                            };
                        });
                    }

                    if ($('.additional-service-doorcode', selected).length) {
                        $('.additional-service-doorcode', selected).each(function(i, object) {
                            var helpText = '';
                            if ($('.addon-description', $(object).parent()).length) {
                                helpText =
                                    $('.addon-description', $(object).parent()).text();
                            }
                            var id = $(object).attr('name');
                            var name = '';
                            if ($('.name', $(object).parent()).length) {
                                name =
                                    $('.name', $(object).parent()).text();
                            }
                            var value = $(object).attr('value');
                            if (value !== '') {
                                additionalServices[id] = {
                                    helpText: helpText,
                                    id: id,
                                    name: name,
                                    value: value
                                };
                            }
                        });
                    }

                    var session = {
                        additionalServices: additionalServices,
                        carrierTitle: carrierTitle,
                        methodTitle: methodTitle,
                        shipmentOptionId: shipmentOptionId,
                        servicePointId: servicePointId,
                        servicePointName: servicePointName
                    };
                    mspostnord_debug(function() {
                        console.log('session:');
                        console.log(session);
                    });
                    if (session.shipmentOptionId
                        && session.carrierTitle
                        && session.methodTitle
                       ) {
                        $.ajax(
                            mspostnord_ajaxUrl,
                            {
                                data: {
                                    action:
                                    'save',
                                    orderId:
                                    mspostnord_orderId,
                                    session:
                                    session
                                },
                                dataType: 'json',
                                error: function (response) {
                                    mspostnord_isSaving = false;
                                    $('#mspostnord-information-admintml-order-view').removeClass('is-loading');
                                },
                                success: function(response) {
                                    mspostnord_isSaving = false;
                                    $('#mspostnord-information-admintml-order-view').removeClass('is-loading');
                                    if (typeof(response.updated) !== 'undefined'
                                        && response.updated > 0
                                       ) {
                                        window.location.reload();
                                    }
                                },
                                method: 'GET'
                            }
                        );
                    } else {
                        alert(mspostnord_i18n['Missing required data to save!']);
                        mspostnord_isSaving = false;
                        $('#mspostnord-information-admintml-order-view').removeClass('is-loading');
                    }
                } else {
                    alert(mspostnord_i18n['Please select a shipment option first!']);
                    mspostnord_isSaving = false;
                    $('#mspostnord-information-admintml-order-view').removeClass('is-loading');
                }
            }
        }

        /**
         * If debug is on, run callback
         * @param {function} callback
         */
        function mspostnord_debug(callback) {
            if (window.mspostnord_is_debug) {
                console.log('Mediastrategi PostNord Ship Connect Debug Start');
                callback();
                console.log('Mediastrategi PostNord Ship Connect Debug End');
            }
        }

        // Check the DOM for necessary information
        if ($('#mspostnord-information-admintml-order-view').length) {
            mspostnord_ajaxUrl =
                $('#mspostnord-information-admintml-order-view').attr('data-ajax-url');
            if (typeof(mspostnord_ajaxUrl) === 'undefined'
                || mspostnord_ajaxUrl === ''
               ) {
                console.error('Failed to determine ajax URL');
            }
            mspostnord_orderId =
                $('#mspostnord-information-admintml-order-view').attr('data-order-id');
            if (typeof(mspostnord_orderId) === 'undefined'
                || mspostnord_orderId === ''
               ) {
                console.error('Failed to determine order id');
            }

            if ($('#mspostnord-order-i18n').length) {
                var decodedi18n = false;
                try {
                    decodedi18n =
                        $.parseJSON($('#mspostnord-order-i18n').text());
                } catch (e) {
                    decodedi18n = false;
                }
                if (decodedi18n) {
                    mspostnord_i18n =
                        decodedi18n;
                }
            }

            window.mspostnord_is_debug =
                $('#mspostnord-information-admintml-order-view.is-debug').length > 0;
            if (mspostnord_ajaxUrl && mspostnord_orderId) {
                mspostnord_debug(function() {
                    console.log('Initialized');
                });
                $('button.action-search-method').click(function(event) {
                    event.preventDefault();
                    mspostnord_refresh();
                });
                $('button.action-search-save').click(function(event) {
                    event.preventDefault();
                    mspostnord_save();
                });
                $('button.action-search-save').attr('disabled', 'disabled');
            } else {
                mspostnord_debug(function() {
                    console.log('Failed to find necessary information');
                });
            }
        }
    });
});
